'' =================================================================================================
''
''   File....... jm_sd_directory.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1    = 31                                                   ' programming / terminal
  TX1    = 30
  
  SDA    = 29                                                   ' eeprom / i2c
  SCL    = 28

  SD_CS  =  3
  SD_DI  =  2
  SD_CLK =  1
  SD_DO  =  0

  

con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"                                   ' terminal IO
  sd   : "fsrw"                                                 ' file IO
 

var

  byte  filecount                                               ' files on SD card (max = 100)
  byte  filenames[13 * 100]                                     ' space for 100 8.3 filenames


dat

Banner                  byte    "File            Size      ", CR
                        byte    "------------    ----------", CR
                        byte    0
                        

pub main | check

  term.start(RX1, TX1, %0000, 115_200)
  pause(10)

  check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)
  if (check == 0)
    directory
    
  else
    term.tx(CLS)
    term.str(string("ERROR: sd card failed to mount"))

  repeat
    waitcnt(0)


pub directory | fcount, check, idx, p_fname, fsize

  ' populate filenames list first

  bytefill(@filenames, 0, 13 * 100)                                     ' clear list

  sd.opendir                                                            ' setup for .nextfile
  fcount := 0                                                           ' reset files count
 
  repeat
    check := \sd.nextfile(@filenames[13 * fcount])                      ' get next file in root
    if (check == 0)                                                     ' if found
      fcount += 1                                                       '  increment file count
    else                                                                ' else
      quit                                                              '  done

  ' iterate through list, show name and size

  term.tx(CLS)

  if (fcount > 0)
    term.str(@Banner)
    repeat idx from 0 to fcount-1                                       ' loop through files
      p_fname := @filenames[13 * idx]                                   ' point to name 
      fsize := open_file(p_fname, "r")                                  ' open, get size
      term.str(p_fname)                                                 ' print name
      term.tx(GOTOX)                                                    ' move cursor
      term.tx(16)
      if (fsize > 0)
        term.dec(fsize)                                                 ' print size
      else
        term.tx("?")
      term.tx(CR)

    term.tx(CR)
    term.dec(fcount)
    term.str(string(" file(s) found."))
      
  else
    term.str(string("No files found."))

  sd.pclose


pub open_file(p_fname, mode) | check, fsize

'' Attempt to open file
'' -- p_fname is pointer to filename (z-string)
'' -- mode is single character, (e.g., "r" for read)
'' -- sd card must be mounted before open
'' -- returns file size if able to open
'' -- returns 0 if error

  check := \sd.popen(p_fname, mode)                             ' attempt to open
  if (check => 0)                                               ' if file exists
    fsize := sd.get_filesize                                    '  get size
    if ((mode == "r") or (mode == "R"))                         '  ir read mode
      \sd.seek(0)                                               '   force to beginning of file
    return fsize
  else
    return 0


pub close_file

'' Closes open file

  \sd.pclose


con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}